#import "WriterViewController.h"
#import "Constants.h"
#import "CoreDataStack.h"
#import "LogDataSource.h"
#import "Log.h"

@interface WriterViewController ()
@property(nonatomic,strong) CoreDataStack *coreDataStack;
@property(nonatomic,strong) LogDataSource *logDataSource;
@property(nonatomic) UIBackgroundTaskIdentifier taskIdentifier;
@property(nonatomic, strong) Log *backgroundLog;
@end

@implementation WriterViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    NSURL *storageURL = [[NSFileManager defaultManager] containerURLForSecurityApplicationGroupIdentifier:AppGroupIdentifier];
    storageURL = [storageURL URLByAppendingPathComponent:DatabaseFilename];
    self.coreDataStack = [[CoreDataStack alloc] initWithStorageURL:storageURL];
    self.coreDataStack.sendsUpdates = YES;
    self.logDataSource = [[LogDataSource alloc] initWithTableView:self.tableView context:self.coreDataStack.context];
    self.tableView.dataSource = self.logDataSource;
    
    self.taskIdentifier = UIBackgroundTaskInvalid;
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(beginBackgroundActivity) name:UIApplicationDidEnterBackgroundNotification object:nil];

}

- (IBAction)add:(UIBarButtonItem *)sender
{
    Log *log = [NSEntityDescription insertNewObjectForEntityForName:@"Log" inManagedObjectContext:self.coreDataStack.context];
    log.timestamp = [NSDate date];
    log.message = @"New message";
    [self.coreDataStack save];
}

- (void)beginBackgroundActivity
{
    self.taskIdentifier = [[UIApplication sharedApplication] beginBackgroundTaskWithName:@"CoreDataUpdates" expirationHandler:^{
        if (self.taskIdentifier != UIBackgroundTaskInvalid) {
            [[UIApplication sharedApplication] endBackgroundTask:self.taskIdentifier];
            self.taskIdentifier = UIBackgroundTaskInvalid;
        }
    }];
    
    [self performSelector:@selector(backgroundActivity1) withObject:nil afterDelay:5];
}

- (void)backgroundActivity1
{
    Log *log = [NSEntityDescription insertNewObjectForEntityForName:@"Log" inManagedObjectContext:self.coreDataStack.context];
    log.timestamp = [NSDate date];
    log.message = @"Message added while in background";
    [self.coreDataStack save];
    self.backgroundLog = log;
    [self performSelector:@selector(backgroundActivity2) withObject:nil afterDelay:5];
}

- (void)backgroundActivity2
{
    self.backgroundLog.message = @"Message updated in background, will delete";
    [self.coreDataStack save];
    [self performSelector:@selector(backgroundActivity3) withObject:nil afterDelay:5];
}

- (void)backgroundActivity3
{
    [self.backgroundLog.managedObjectContext deleteObject:self.backgroundLog];
    self.backgroundLog = nil;
    [self.coreDataStack save];
    [self endBackgroundTask];
}

- (void)endBackgroundTask
{
    if (self.taskIdentifier != UIBackgroundTaskInvalid) {
        [[UIApplication sharedApplication] endBackgroundTask:self.taskIdentifier];
        self.taskIdentifier = UIBackgroundTaskInvalid;
    }
}

@end
